function density=gbd_pdf(x,beta);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GBD_PDF - Calculates the values of the probability density
% function (PDF) of a distribution from the Generalized Beta Distribution.
%
% Input:
%   x    = the realization matrix (sample values)
%   beta = the distribution parameters vector [beta(1) beta(2) beta(3) beta(4)]
%
% Output:
%   cp   = the PDF matrix corresponding to x
%
% Copyright (c) Helsinki University of Technology,
% Signal Processing Laboratory,
% Jan Eriksson, Juha Karvanen, and Visa Koivunen.
%
% For details see the files readme.txt
% and gpl.txt provided within the same package.
%
% Version 1.0 (05.09.2000):
% Released by: Juha Karvanen,
%   Helsinki University of Technology
%
% Version 1.1 (26.11.2007):
% Contributed by: Milton Woods,
%   CSIRO Marine and Atmospheric Research (Canberra, Australia).
% Changes from version 1.0:
% - Set pdf to zero for x outside the range [beta(1),beta(1)+beta(2)],
%   as required by the definition of the GBD.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
if nargin<2, disp('Usege: gbd_pdf(x,beta)');
             error('Not enough input arguments.');  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The PDF is given by the equation (20) in
%   Eriksson, J., Karvanen, J., and Koivunen, V.:
%   "Source Distribution Adaptive Maximum Likelihood Estimation of ICA Model", 
%   Proceedings of Second International Workshop on
%   Independent Component Analysis and Blind Signal Separation,
%   Helsinki 2000, pp. 227--232
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
C=gamma(beta(3)+beta(4)+2)/(gamma(beta(3)+1)*gamma(beta(4)+1));
ix=find( (beta(1)<=x) & (x<=(beta(1)+beta(2))) );
density=zeros(size(x));
density(ix)=C*beta(2)^(-(beta(3)+beta(4)+1))*(x(ix)-beta(1)).^beta(3).*...
	(beta(1)+beta(2)-x(ix)).^beta(4);
% The end of the function %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
