function [v,x,smom,A,s,slambda,sbeta]=ic_rnd(vcorrcoef,fircoef,vmu1,vmu2,vskew,vkurt,N,U);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% IC_RND    v. 1.1
% This program generates correlated non-Gaussian random variables
% using the method described in
%
% [1] J. Karvanen, Generation of Correlated Non-Gaussian Random Variables 
%     from Independent Components, to appear in ICA2003. 
%     available from http://wooster.hut.fi/~jkarvane/
%
%
% The input variables are: (m = number of components to be generated)
% vcorrcoef (m*m) = matrix of correlation coefficients for the output v, 
%                   (positive definitive, all elements between -1 and +1,
%                   diagonal elements = 1)
% fircoef (m*max filter length) = FIR filter coefficients 
%                                 The row i contains the filter applyed for the
%                                 component i after the instantaneous mixing.
% vmu1 (m*1) = mean of the output v 
% vmu2 (m*1) = variance of the output v 
% vskew (m*1) = skewness of the output v 
% vkurt (m*1) = kurtosis of the output v (Gaussian=3) 
% N = number of samples to be generated 
% U (optional)(m*m) = orthogonal rotation matrix 
%
% The output variables are:
% v (m*N) = the output with the statistics specified in vcorrcoef, vmu1, vmu2,
%                 vskew and vkurt. The temporal structure of v is specified by the 
%                 filter coefficients in fircoef.
% A (m*m) = mixing matrix for the instantaneous mixing
% smom (m*4) = theoretical mean, variance, skewness and kurtosis of sources s
% s (m*N) = sources
% slambda (m*4) = the parameters used for the GLD
% sbeta (m*4) = the parameters used for the GBD
% x (m*N) = sources after the instantaneous mixing, before filtering
% 
% See the paper [1] for further information.
%
%
% Example:
% generates data for the first example in the paper [1]:
% vcorrcoef=[1.0 0.7 0.3 -0.2 -0.2; ...
%            0.7 1.0 0.2 -0.2 -0.2; ...
%            0.3 0.2 1.0 -0.3 -0.2; ...
%           -0.2 -0.2 -0.3 1  0.4; ...
%           -0.2 -0.2 -0.2 0.4 1];
% fircoef=[1 1 1 1 1]'; %no temporal structure
% vmu1=[0 0 0 0 0]';
% vmu2=[1 1 1 1 1]';
% vskew=[0.6 -0.4 0 0.4 0.2]';
% vkurt=[5 4 3 2 1.5]';
% v=ic_rnd(vcorrcoef,fircoef,vmu1,vmu2,vskew,vkurt,200)
%
%
% The EGLD sources are generated using the EGLD functions
% released as a part of EGLD-ICA. 
% See http://www.tsi.enst.fr/icacentral/algos.html
% and http://wooster.hut.fi/statsp/papers/EGLD_ICA.zip
%
% BETARND from Matlab Statistics toolbox is used for the generation
% of Beta distributed random variables.
%
% FILTER from Matlab Signal Processing Toolbox is used for filtering.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (c) 
% Helsinki University of Technology
% Juha Karvanen
%
% For details see the files readme.txt
% and gpl.txt provided within the same package.
%
% Version 1.0 (20.01.2003):
% Released by: Juha Karvanen,
%   Helsinki University of Technology
%
% Version 1.1 (26.11.2007):
% Contributed by: Milton Woods,
%   CSIRO Marine and Atmospheric Research (Canberra, Australia).
% Changes from version 1.0:
% - moved call to egld_use into gld_momentfit.
% - fixed equations for 4th moments (xmu4 & smu4)
%   so that they match equations 9 and 15 in paper [1].
% - replaced tests of equality with tests of near-equality
%   to allow for small numerical errors.
% - changed output argument smu to smom,
%   which contains mean, variance, skewness and kurtosis
%   rather than raw moments of the random sources.
% - find mixing matrix A from covariance matrix rather than
%   correlation matrix of x, for consistency with paper [1].
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The parameters for the selection between the GLD and the GBD are set.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
borderBase=2.2;
borderSlope=1.75;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The sizes of the input matrices are checked.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<7,
   error('Seven or eight arguments are needed. (vcorrmatrix,fircoef,vmu1,vmu2,vskew,vkurt,N,U)')
end
if (N<0 | N~=round(N)),
   error('N must be a positive integer.')
end
m=size(vcorrcoef,1);
if (size(vcorrcoef,2)~=m | ndims(vcorrcoef)>2),
   error('vcorrcoef must be a square matrix.')
end
if size(vmu1,1)~=m,
   error('The number rows in vmu1 and vcorrcoef must agree.')
end
if size(vmu2,1)~=m,
   error('The number rows in vmu2 and vcorrcoef must agree.')
end
if size(vskew,1)~=m,
   error('The number rows in vskew and vcorrcoef must agree.')
end
if size(vkurt,1)~=m,
   error('The number rows in vkurt and vcorrcoef must agree.')
end
if size(fircoef,1)~=m,
   error('The number rows in fircoef and vcorrcoef must agree.')
end
if nargin==7,
   U=eye(m);
elseif (size(U,1)~=m | size(U,2)~=m),
   error('U must be a m*m square matrix.')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The suitability of the specified moments, correlation coefficients
% and rotation matrix are checked.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
abstol=10*eps;
if any(any(abs(U*U'-eye(m))>abstol)) | ...
   any(any(abs(U'*U-eye(m))>abstol))
   error('Matrix U must be orthogonal.')
end
if min(vmu2)<=0,
   error('Variances given in vmu2 must be positive.')
end
if min(vkurt)<=1,
   error('Kurtosis given in vkurt must be greater than 1.')
end
if any(any(abs(vcorrcoef-vcorrcoef')>abstol)) | ...
   (min(eig(vcorrcoef))<=0),
   error('vcorrcoef must be positive definite.')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The statistics for x are computed from v and FIR-coefficients.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
xmu1=vmu1./sum(fircoef,2);
xmu2=vmu2./sum(fircoef.^2,2);
vmu3=(vmu2.^(3/2)).*vskew;
vmu4=(vmu2.^2).*vkurt;
xmu3=vmu3./sum(fircoef.^3,2);
xmu4=(vmu4-3*vmu2.^2)./sum(fircoef.^4,2)+3*xmu2.^2;

vcovmatrix=(sqrt(vmu2*vmu2')).*vcorrcoef;
firmatrix=fircoef*fircoef';
xcovmatrix=vcovmatrix./firmatrix;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Mixing matrix A is solved using matrix square root.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
A=real(sqrtm(xcovmatrix));
A=A*U;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The statistics for s are computed from x and A.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
smu1=inv(A)*xmu1;
smu2=inv(A.^2)*xmu2;
smu3=inv(A.^3)*xmu3; 
smu4=inv(A.^4)*(xmu4-3*xmu2.^2)+3*smu2.^2;
sskew=smu3./(smu2.^(3/2));
skurt=smu4./(smu2.^2);
smom=[smu1 smu2 sskew skurt];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The suitability of the source moments is checked. See equation (10) in [1].
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if min(smu2)<=0,
   disp('Source mean, variance, skewness and kurtosis (in rows):')
   disp(num2str(smom))
   error('Non-positive source variances. Random variables cannot be generated. Adjust the input variables.')
end
for i=1:m,
   testmatrix=[1 0 smu2(i); ...
               0 smu2(i) smu3(i); ...
               smu2(i) smu3(i) smu4(i)];
   if det(testmatrix)<0,
     disp('Source mean, variance, skewness and kurtosis (in rows):')
     disp(num2str(smom))
     error(['Impossible source moments (row ' int2str(i) '). Random variables cannot be generated. Adjust the input variables.'])
   end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sources are generated.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
s=zeros(m,N);
slambda=zeros(m,4);
sbeta=zeros(m,4);
for i=1:m,
   if skurt(i) > (borderBase+borderSlope*sskew(i)^2),
   	slambda(i,:)=gld_momentfit(sskew(i),skurt(i),smu1(i),smu2(i));
   	s(i,:)=gld_rnd(slambda(i,:),[1,N]);
   else
      sbeta(i,:)=gbd_momentfit(sskew(i),skurt(i),smu1(i),smu2(i));
      s(i,:)=gbd_rnd2(sbeta(i,:),[1,N]);
%      s(i,:)=gbd_rnd(sbeta(i,:),[1,N]); % Faster, but uses more memory
   end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sources are mixed and filtered.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x=A*s;
for i=1:m,
   v(i,:)=filter(fircoef(i,:),1,x(i,:));
end

