function lambda = gld_momentfit(alpha3,alpha4,alpha1,alpha2)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GLD_MOMENTFIT - Estimates the parameters of the Generalized 
% Lambda Distribution using the first four moments (the method of moments). 
%
% Input
%   alpha3 = sample 3rd moment
%   alpha4 = sample 4th moment
%   alpha1 = sample mean       (optional)
%   alpha2 = sample variance   (optional)
%  
% Output
%   lambda = the distribution parameters vector  
%            [lambda(1),lambda(2),lambda(3),lambda(4)]
%
% Parameters are linearly interpolated between the values 
% provided by the GLD_moment_table. The table is adapted from
%   E.J. Dudewicz, Z.A. Karian: "The Extended Generalized Lambda
%   Distribution (EGLD) System for Fitting Distributions to Data with 
%   Moments, II: Tables", American J. of Math. and Management
%   Sciences, 1996, Vol. 16, NOS. 3& 4, 271--332
%
% Copyright (c) Helsinki University of Technology,
% Signal Processing Laboratory,
% Jan Eriksson, Juha Karvanen, and Visa Koivunen.
%
% For details see the files readme.txt
% and gpl.txt provided within the same package.
%
% Version 1.0 (11.09.2002):
% Released by: Juha Karvanen,
%   Helsinki University of Technology
%
% Version 1.1 (26.11.2007):
% Contributed by: Milton Woods,
%   CSIRO Marine and Atmospheric Research (Canberra, Australia).
% Changes from version 1.0:
% - Changed table format to 3D array,
%   with columns corresponding to fixed alpha3.
% - Simplified interpolation scheme to exploit new table structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check inputs:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<2,  disp('Usage: gld_momentfit(alpha3,alpha4) or');
              disp('gld_momentfit(alpha3,alpha4,alpha1,alpha2)');
              error('Not enough input arguments.');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% If alpha1 and alpha2  are not given, the default values alpha1=0
% and alpha2=1 are substituted.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin==2
  alpha1=0;
  alpha2=1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load look-up table.
% Note that egld_use may have been called previously -
% re-load only if the table appears to be missing.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global GLD_moment_table
if isempty(GLD_moment_table)
  egld_use;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Find table cell containing abs(alpha3),alpha4.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Find left and right alpha3 values:
il=max(find(GLD_moment_table(1,:,1)<=abs(alpha3)));
ir=min(find(GLD_moment_table(1,:,1)>abs(alpha3)));

if ~( isempty(il) | isempty(ir) )
  % Interpolate tabulated alpha4 values to alpha3:
  da3=GLD_moment_table(1,ir,1)-GLD_moment_table(1,il,1);
  da3r=GLD_moment_table(1,ir,1)-abs(alpha3);
  a4i=(da3r/da3)*GLD_moment_table(:,il,2) + ...
      (1-da3r/da3)*GLD_moment_table(:,ir,2);

  % Find values of interpolated alpha4 at the top and bottom
  % of the table cell containing the specified (alpha3,alpha4):
  ib=max(find(a4i<=alpha4));
  it=min(find(a4i>alpha4));
else
  ib=[];
  it=[];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check if the moment values are covered by the table.
% If not, the closest point is used.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isempty(il) | isempty(ir) | isempty(it) | isempty(ib)
  disp(['Warning: The moment values, alpha3=',num2str(alpha3),...
		    ', alpha4=',num2str(alpha4),',']);
	      disp('  cannot be found by interpolation of the GLD tables.');
	      disp('  The closest table entry is used instead.');

  [da3m,ia3m]=min(abs(GLD_moment_table(1,:,1)-abs(alpha3)));
  [da4m,ia4m]=min(abs(GLD_moment_table(:,ia3m,2)-alpha4));
  lambda=GLD_moment_table(ia4m,ia3m,3:6);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The lambda vector is obtained by linear interpolation
% within the table cell found above. This vector is
% for the standarized data (mean=0, variance=1).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
else

% Interpolate lambda to specified abs(alpha3)
% along top and bottom edges of cell:
  lambdat=(da3r/da3)*GLD_moment_table(it,il,3:6) + ...
          (1-da3r/da3)*GLD_moment_table(it,ir,3:6);
  lambdab=(da3r/da3)*GLD_moment_table(ib,il,3:6) + ...
          (1-da3r/da3)*GLD_moment_table(ib,ir,3:6);

% Interpolate lambda from the top and bottom of the cell
% to the specified alpha4:
  da4=a4i(it)-a4i(ib);
  da4t=a4i(it)-alpha4;
  lambda=(da4t/da4)*lambdab + (1-da4t/da4)*lambdat;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now alpha1, alpha2 and the sign of alpha3 are taken into account.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if sign(alpha3)==-1
  lambda=[-lambda(1) lambda(2) lambda(4) lambda(3)];
end

lambda=[lambda(1)*sqrt(alpha2)+alpha1,lambda(2)/sqrt(alpha2),lambda(3),lambda(4)];

% The end of the function %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
